

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% BLS.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function [point,step,info]=BLS(fun,point,step,tune,info);
% find a step size alpha with mu(alpha)|mu(alpha)-1| >= beta
%
% for details of input and output structures see lmbopt.m


function [point,step,info]=BLS(fun,point,step,tune,info)

% regularized directional derivative
step.gTp=step.pI'*point.gI;
step.gTp= min(step.gTp,-tune.Deltareg*(abs(point.gI)'*abs(step.pI)));

% full search direction
step.p=zeros(point.n,1); 
step.p(point.I)=step.pI;


if info.ng==1, point.df = point.Deltaf;
else
  absf = abs(point.f);
  if mod(info.ng,tune.mdf)==0, point.df=point.Deltaf*(absf+1);
  else, point.df=max(point.Df);
  end
  clear absf
end


if isfield(tune,'exact'), exact=tune.exact;
else exact=0;
end;


% check consistency of input
if ~(step.gTp<0),
 step.gTp;
 error('step.gTp <0 is required')
end;

if ~(point.df>0), error('point.df>0 required'); end;

while 1,
  if point.f+point.df>point.f, break; end;
  point.df=10*point.df;
end;

% compute robust initial step
[step]=goodStep(point,step,tune);

% assign initial step size
if isfield(tune,'ainit'),
  % use given initial step
  step.anew=tune.ainit;
else  
  % use robust initial step
  step.anew=step.agood;
end;


% define bracket [alow,aupp] for step sizes 
alow=0;
aupp=inf;

% initialize lists
alist(1)=0;  % step sizes 
dlist(1)=0;  % differences in function values
info.eff=NaN;
descent=0;
rob=0;
nf=0;
while 1,
    
  % evaluate function at current step
  point.fnew=fun.f(max(point.low,min(point.x+step.anew*...
             step.p,point.upp)));
  nf=nf+1;
  if isnan(point.fnew), point.fnew=inf; end;
  dnew=point.fnew-point.f;
  alist(nf+1)=step.anew;
  dlist(nf+1)=dnew;

  % quit line search?
  mu=dnew/(step.anew*step.gTp);
  if mu*abs(mu-1)>=tune.beta | info.eff==1,
    % line search efficient
    info.eff=1;
    if info.prt>=1, disp('BLS: line search efficient'); end
    if ~exact, 
      break; 
    end;
  elseif nf>1 & ~descent & rob>0
    % robust nonmonotone step accepted
    info.eff=2;
     if info.prt>=1, disp('BLS: robust nonmonotone step accepted'); end
    break;
  elseif nf>=tune.lmax,
    % limit on function values reached
    if descent,
      % descent
      info.eff=3;
      if info.prt>=1, 
          disp(['BLS: limit on function values reached',...
                ' with improvement on f']); 
      end
      break;
    else
      % no descent
      info.eff=4;
      if info.prt>=1, 
        disp(['BLS: limit on function values reached',...
            ' without improvement on f']); 
      end
      break;
    end;
  end;

  % update bracket
  if descent,
    % update bracket for descent
    if mu>0.5,
      alow=step.anew;
    else
      aupp=step.anew;
    end;
  elseif dnew<0, 
    % first descent step
    descent=1;
    % create bracket for descent
    % lower part
    ind=find(dlist<0);
    alow=max(alist(ind));
    rob=-1;
    % upper part
    ind=find(dlist>=0 & alist>alow);
    if isempty(ind), aupp=inf;
    else             aupp=min(alist(ind));
    end;
  else
    % no descent; update robust bracket
    if dnew<=point.df,
      alow=step.anew;
      rob=dnew;
    else
      aupp=step.anew;
    end;
  end;

  % determine next step
  if nf==1,
    % first step
    if mu<1, 
      % secant step for Goldstein quotient
      step.anew=step.anew*0.5/(1-mu); 
      if step.anew==0,step.anew=step.amin; end;
    else
      % extrapolation
      step.anew=max(step.amin,step.anew*tune.q);
    end;
    exact=0; 
  else
    % robust bisection 
    if aupp==inf, 
      % extrapolation
      step.anew=step.anew*tune.q;
    elseif alow==0,
      % contraction
      step.anew=step.anew/tune.q;
    else
       alow0 = max(alow,step.amin); 
        switch tune.bis
            case 0 % geometric mean bisection
                step.anew  = alow0*sqrt(aupp/alow0);
            case 1 % cubic bisection
                step.anew  = alow*(aupp/alow0)^(1/3);
            case 2
                  if mod(nf,tune.mbis)==0 % geometric mean bisection
                    step.anew  = alow0*sqrt(aupp/alow0);
                  else                    % cubic bisection
                    step.anew  = alow*(aupp/alow0)^(1/3);
                  end
            otherwise
                disp('tune.bis should be 0,1,2')
                next;
        end
    end;
  end; % of if nf

end; % of while 

% return robust step 

[dbest,irob]=min(dlist);
if dbest<0,
  % improvement
  step.anew=alist(irob);
  point.fnew=point.f+dlist(irob);
  info.nf = info.nf+nf;
  info.nf2g=info.nf+2*info.ng;
  clear alist dlist ind alow0 aupp alow nf mu inew irob
  return;
else

    % treat failed line search (no improvement)
    % find point with smallest robust change

    % check for robust change
    ind=find(dlist>0 & isfinite(dlist));
    [dnew,inew]=min(dlist(ind)); % empty if ind is empty
    irob=ind(inew);
    if dnew<=point.df,
      % accept point with robust change (never the case if dnew is empty)
    elseif isempty(dnew) | any(dlist<=0),
      % function almost flat; take step with largest d
      ind=find(isfinite(dlist));
      [dnew,inew]=max(dlist(ind));
      irob=ind(inew);
      if dnew>0, % never the case if dnew is empty
      else
        % function is flat; take largest step
        [~,inew]=max(alist(ind));
        irob=ind(inew);
      end;
    else
      if dnew<=tune.Deltar*point.df,
        % accept point with nonrobust change (at most twice too large)
        % this factor could be changed; >50 is unstable
      else
        % take largest almost flat step 
        ind=find(dlist<=point.df);
        [~,inew]=max(alist(ind));
        irob=ind(inew);
      end;
    end;

  step.anew=alist(irob);
  point.fnew=point.f+dlist(irob);
  info.nf = info.nf+nf;
  info.nf2g=info.nf+2*info.ng;
  
  clear alist dlist ind alow0 aupp alow nf mu ind inew irob

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   



 
