

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%% driverLMBOPT.m %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file illustrates the use of LMBOPT
% by showing how to minimize the function f(x):=||Ax-b||_p^e
% for p=2,e=1 in dimension n=300 from a random starting point.
% The goal assumed in the example is the reduction of the initial 
% objective function value by a factor of 0.01 with at most nfmax=200*n 
% function evaluations, to return when one of the two limiting 
% conditions is first satisfied.

clear;

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% create problem definition

% define problem parameters (to be adapted to your problem)
n=1000;          % dimension


cas=2; % 1: linear least square problem, 
       % 2-4: nonlinear unconstrained optimization problem
switch cas
    case 1
    % create random matrix and right hand side for objective function
    % (specific to the model problem; replace by whatever data you
    % need to provide to your objective function)
    A = rand(n)-0.5; b = -sum(A,2);
    % This toy example of a linear least square problem has as solution 
    % the all -1 vector.
    prob.A=A; prob.b=b; 
    prob.name='linear least square';
    case 2  % Chained wood
         prob.n=n; prob.name='Chained wood';
    case 3  %  Brent
         prob.n=n; prob.name='Brent';
    case 4  % Variational 1
         prob.n=n;  prob.name='Variational 1';
end
     
prob.cas=cas;

% Define function values and gradient by subprograms 
% provided in analogy to the following

fun.fg = @(x) funfg(x,prob); 
fun.f  = @(x) funf(x,prob); 
fun.g  = @(x) fung(x,prob);  

% To solve your own problem, simply replace in these lines 
% the expression after @(x) by your expression or function call. 
% Parameters in this expression other than x are passed by value, 
% hence are fixed during minimization. 

% starting point
nu  = 2;
for i=1:n, x(i)=(-1)^i*(nu/(i+nu)); end
x=x(:);

% start and stop info
nfmax   = 200*n;          % stop after nfmax function evaluations
ngmax   = 100*n;          % stop after ngmax gradient evaluations
nf2gmax = nfmax+2*ngmax;
secmax  = inf;           % stop after secmax seconds
upp     = 100*ones(n,1); % lower bound
low     = -upp;          % upper bound
epsilon = 1e-6;   % absolute accuracy requested for reduced gradient

% customization
prt=0;        % print level 
              %   -1: nothing, 0: litte, >=1: more and more
Tuning=0;     % 0: no tuning of LMBOPT (recommended at first)
              % 1: full tuning (for specialists only)

% problem definition complete
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% define stop and print criteria
% (indefinite run if no stopping criterion is given)
st.secmax  = secmax;       % (default: inf)
st.nf2gmax = nf2gmax;
st.prt     = prt;          % (default: -1)
st.epsilon = epsilon;

if prt>=0, st.time2print = cputime+1; end
   
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% solve the problem with LMBOPT

tic; % set clock

if ~Tuning,
  tune=[];
else % set tuning parameters (for specialists only)
  % given are the defaults
  % only the deviating parameters need to be set!
  
  % how many stucks before taking special action?, >0
  tune.nsmin = 1;
  % number of local steps before CG is started
  tune.nwait = 1; 
  % restart after rfac*I local steps
  tune.rfac = 2.5; 
  % number of local steps before freeing is allowed
  tune.nlf = 2; 
  % tiny factor for regularizing Deltaf if not monotone 
  tune.Deltam = 1e-13; 
  % tiny factor for regularizing gTp
  tune.Deltapg = 1e-12; 
  % tiny regularization factor for subspace Hessian, <<1
  tune.DeltaH = eps; 
  % tiny factor for initial step
  tune.Deltaalpha = 5*eps;
  % iteration limit in efficient line search
  tune.lmax = 3; 
  % threshold for determining efficiency (0<beta<=0.2)
  tune.beta = 0.02; 
  % bisection (1: geometric mean, 2: cubic)
  tune.bis = 0;
  if tune.bis==2, tune.mbis=20; end
  % threshold for efficiency of CG (0<betaCG<0.25)
  tune.betaCG = 0.001; 
  % initial scaling step?
  tune.scale = 1; 
  % regularization angle
  tune.Deltaangle = 1e-12; 
  % factor for regularization
  tune.Deltareg = 1e-12; 
  % for generating w>0 in enforceAngle.m
  tune.Deltaw = eps; 
  % relative accuracy of f in first step
  tune.facf = 1e-8; 
  % tiny factor for interior move, <<1
  tune.Deltax = 1e-12; 
  % subspace dimension
  tune.m = 12; 
  % length Df
  tune.mf = 2; 
  % parameter for Df
  tune.mdf = 20; 
  % choose update formula (0 or 1)
  tune.typeH = 0; 
  % iteration limit in null steps
  tune.nnullmax = 3; 
  % factor for null step
  tune.del = 1e-10; 
  % factor for finding almost flat step
  tune.Deltar = 20; 
  % factor for adjusting gradient
  tune.Deltag = 100; 
  % tiny factor for breakpoint
  tune.Deltab = eps; 
  % factor for upp
  tune.Deltau = 1000; 
  % parameter for adjusting direction
  tune.theta = 0.85; 
  % enforce exact line search on quadratic
  tune.exact = 0; 
  % lower bound for BB parameter
  tune.lammin = 1e-30; 
  % upper bound for BB parameter
  tune.lammax = 1e+30; 
  % gradient tolerance for skipping update (used in Powell condition)
  tune.Deltapo = eps; 
  % iteration limit in number of stuck
  tune.nstuckmax = inf; 
end
  
% call LMBOPT
[x,f,info] = LMBOPT(fun,x,low,upp,tune,st);


if prt>=0    
% display output
disp('================================================================')

if n<=100, disp('best point found: xbest=');  x', end % best point found  
disp('================================================================')

disp(['function value at xbest: f(xbest)=', num2str(f)]); 
disp(['nf+2*ng: ', num2str(info.nf2g)]);
secused =cputime-info.initTime;
disp(['time used up to now: ', num2str(secused)]);
disp(['reduced gradient at xbest: grad(xbest)=', num2str(info.acc)]);
disp('================================================================')
info  % progress report by LMBOPT
disp('================================================================')
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

